/* vim: set shiftwidth=2 tabstop=8 autoindent cindent expandtab: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

interface nsIDOMMediaQueryListListener;

/* see http://dev.w3.org/csswg/cssom-view/#the-mediaquerylist-interface */

[scriptable, uuid(e0e49c52-915b-40f9-9cba-6026305cdf3e)]
interface nsIDOMMediaQueryList : nsISupports
{
  readonly attribute DOMString media;
  readonly attribute boolean matches;

  void addListener(in nsIDOMMediaQueryListListener listener);
  void removeListener(in nsIDOMMediaQueryListListener listener);
};

[scriptable, function, uuid(279a5cbd-5c15-475d-847b-e0de1624eb77)]
interface nsIDOMMediaQueryListListener : nsISupports
{
  void handleChange(in nsIDOMMediaQueryList mql);
};
